@@ -38,7 +38,7 @@ class CiviCRM_For_WordPress_Shortcodes {
 
   /**
    * @var array
-   * The stored shortcodes.
+   * The stored Shortcodes.
    * @since 4.6
    * @access public
    */
@@ -46,7 +46,7 @@ class CiviCRM_For_WordPress_Shortcodes {
 
   /**
    * @var array
-   * The array of rendered shortcode markup.
+   * The array of rendered Shortcode markup.
    * @since 4.6
    * @access public
    */
@@ -73,22 +73,22 @@ class CiviCRM_For_WordPress_Shortcodes {
   }
 
   /**
-   * Register hooks to handle the presence of shortcodes in content.
+   * Register hooks to handle the presence of Shortcodes in content.
    *
    * @since 4.6
    */
   public function register_hooks() {
 
-    // Register the CiviCRM shortcode.
+    // Register the CiviCRM Shortcode.
     add_shortcode('civicrm', [$this, 'render_single']);
 
-    // Add CiviCRM core resources when a shortcode is detected in the post content.
-    add_action('wp', [$this, 'prerender'], 10, 1);
+    // Pre-render CiviCRM content when one or more Shortcodes are detected.
+    add_action('wp', [$this, 'prerender'], 20, 1);
 
   }
 
   /**
-   * Determine if a CiviCRM shortcode is present in any of the posts about to be displayed.
+   * Determine if a CiviCRM Shortcode is present in any of the posts about to be displayed.
    *
    * Callback method for 'wp' hook, always called from WordPress front-end.
    *
@@ -108,6 +108,11 @@ class CiviCRM_For_WordPress_Shortcodes {
       return;
     }
 
+    // Bail if this is a Favicon request.
+    if (function_exists('is_favicon') && is_favicon()) {
+      return;
+    }
+
     // A counter's useful.
     $shortcodes_present = 0;
 
@@ -123,10 +128,10 @@ class CiviCRM_For_WordPress_Shortcodes {
 
         global $post;
 
-        // Check for existence of shortcode in content.
+        // Check for existence of Shortcode in content.
         if (has_shortcode($post->post_content, 'civicrm')) {
 
-          // Get CiviCRM shortcodes in this post.
+          // Get CiviCRM Shortcodes in this post.
           $shortcodes_array = $this->get_for_post($post->post_content);
 
           // Sanity check.
@@ -135,7 +140,7 @@ class CiviCRM_For_WordPress_Shortcodes {
             // Add it to our property.
             $this->shortcodes[$post->ID] = $shortcodes_array;
 
-            // Bump shortcode counter.
+            // Bump Shortcode counter.
             $shortcodes_present += count($this->shortcodes[$post->ID]);
 
           }
@@ -148,15 +153,23 @@ class CiviCRM_For_WordPress_Shortcodes {
     // Reset loop.
     rewind_posts();
 
+    // Bail if there are no Shortcodes.
+    if ($shortcodes_present === 0) {
+      return;
+    }
+
+    // Set context.
+    $this->civi->civicrm_context_set('shortcode');
+
     // Did we get any?
     if ($shortcodes_present) {
 
-      // We need CiviCRM initialised prior to parsing shortcodes.
+      // We need CiviCRM initialised prior to parsing Shortcodes.
       if (!$this->civi->initialize()) {
         return;
       }
 
-      // How should we handle multiple shortcodes?
+      // How should we handle multiple Shortcodes?
       if ($shortcodes_present > 1) {
 
         // Add CSS resources for front end.
@@ -165,12 +178,12 @@ class CiviCRM_For_WordPress_Shortcodes {
         // Let's add dummy markup.
         foreach ($this->shortcodes as $post_id => $shortcode_array) {
 
-          // Set flag if there are multple shortcodes in this post.
+          // Set flag if there are multple Shortcodes in this post.
           $multiple = (count($shortcode_array) > 1) ? 1 : 0;
 
           foreach ($shortcode_array as $shortcode) {
 
-            // Mimic invoke in multiple shortcode context.
+            // Mimic invoke in multiple Shortcode context.
             $this->shortcode_markup[$post_id][] = $this->render_multiple($post_id, $shortcode, $multiple);
 
           }
@@ -184,7 +197,7 @@ class CiviCRM_For_WordPress_Shortcodes {
         add_action('wp', [$this->civi, 'front_end_page_load'], 100);
 
         /*
-         * Since we have only one shortcode, run the_loop again.
+         * Since we have only one Shortcode, run the_loop again.
          * The DB query has already been done, so this has no significant impact.
          */
         if (have_posts()) {
@@ -199,10 +212,10 @@ class CiviCRM_For_WordPress_Shortcodes {
               continue;
             }
 
-            // The shortcode must be the first item in the shortcodes array.
+            // The Shortcode must be the first item in the Shortcodes array.
             $shortcode = $this->shortcodes[$post->ID][0];
 
-            // Check to see if a shortcode component has been repeated?
+            // Check to see if a Shortcode component has been repeated?
             $atts = $this->get_atts($shortcode);
 
             // Test for hijacking.
@@ -241,11 +254,11 @@ class CiviCRM_For_WordPress_Shortcodes {
 
     }
 
-    // Flag that we have parsed shortcodes.
+    // Flag that we have parsed Shortcodes.
     $this->shortcodes_parsed = TRUE;
 
     /**
-     * Broadcast that shortcodes have been parsed.
+     * Broadcast that Shortcodes have been parsed.
      *
      * @since 4.6
      */
@@ -254,7 +267,7 @@ class CiviCRM_For_WordPress_Shortcodes {
   }
 
   /**
-   * Handles CiviCRM-defined shortcodes.
+   * Handles CiviCRM-defined Shortcodes.
    *
    * @since 4.6
    *
@@ -263,9 +276,9 @@ class CiviCRM_For_WordPress_Shortcodes {
    */
   public function render_single($atts) {
 
-    // Do not parse shortcodes in REST context for PUT, POST and DELETE methods.
+    // Do not parse Shortcodes in REST context for PUT, POST and DELETE methods.
     if (defined('REST_REQUEST') && REST_REQUEST && (isset($_PUT) || isset($_POST) || isset($_DELETE))) {
-      // Return the original shortcode.
+      // Return the original Shortcode.
       $shortcode = '[civicrm';
       foreach ($atts as $att => $val) {
         $shortcode .= ' ' . $att . '="' . $val . '"';
@@ -274,7 +287,7 @@ class CiviCRM_For_WordPress_Shortcodes {
       return $shortcode;
     }
 
-    // Check if we've already parsed this shortcode.
+    // Check if we've already parsed this Shortcode.
     global $post;
     if (is_object($post)) {
       if (!empty($this->shortcode_markup)) {
@@ -288,30 +301,69 @@ class CiviCRM_For_WordPress_Shortcodes {
             $this->shortcode_in_post[$post->ID]++;
           }
 
-          // This shortcode must have been rendered.
+          // This Shortcode must have been rendered.
           return $this->shortcode_markup[$post->ID][$this->shortcode_in_post[$post->ID]];
 
         }
       }
     }
 
-    // Preprocess shortcode attributes.
+    // Preprocess Shortcode attributes.
     $args = $this->preprocess_atts($atts);
 
-    // Sanity check for improperly constructed shortcode.
-    if ($args === FALSE) {
-      return '<p>' . __('Do not know how to handle this shortcode.', 'civicrm') . '</p>';
+    // Check for pathless Shortcode.
+    if (empty($args['q'])) {
+
+      $content = '<p>' . __('This Shortcode could not be handled. It could be malformed or used incorrectly.', 'civicrm') . '</p>';
+
+      /**
+       * Get the markup for "pathless" Shortcodes.
+       *
+       * This filter allows plugins or CiviCRM Extensions to modify the markup used
+       * to display a Shortcode that has no CiviCRM route/path. This may be:
+       *
+       * * Accidental due to an improperly constructed Shortcode or
+       * * Deliberate because a component may not require a route/path
+       *
+       * Used internally by:
+       *
+       * - afform_shortcode_content()
+       *
+       * @since 5.37
+       *
+       * @param str $content The default markup for an improperly constructed Shortcode.
+       * @param array $atts The Shortcode attributes array.
+       * @param array $args The Shortcode arguments array.
+       * @param str Context flag - value is either 'single' or 'multiple'.
+       */
+      return apply_filters('civicrm_shortcode_get_markup', $content, $atts, $args, 'single');
+
     }
 
-    // invoke() requires environment variables to be set.
-    foreach ($args as $key => $value) {
-      if ($value !== NULL) {
-        set_query_var($key, $value);
-        $_REQUEST[$key] = $_GET[$key] = $value;
+    // If there are *actual* CiviCRM query vars, let them take priority.
+    if (!$this->civi->civicrm_in_wordpress()) {
+
+      // Get the Shortcode Mode setting.
+      $shortcode_mode = $this->civi->admin->get_shortcode_mode();
+
+      /** This filter is documented in includes/civicrm.basepage.php */
+      $basepage_mode = (bool) apply_filters('civicrm_force_basepage_mode', FALSE, $post);
+
+      // Skip unless in "legacy mode" or "Base Page mode" is forced.
+      if ($shortcode_mode !== 'legacy' || !$basepage_mode) {
+
+        // invoke() requires environment variables to be set.
+        foreach ($args as $key => $value) {
+          if ($value !== NULL) {
+            set_query_var($key, $value);
+            $_REQUEST[$key] = $_GET[$key] = $value;
+          }
+        }
+
       }
+
     }
 
-    // Kick out if not CiviCRM.
     if (!$this->civi->initialize()) {
       return '';
     }
@@ -319,15 +371,12 @@ class CiviCRM_For_WordPress_Shortcodes {
     // Check permission.
     $argdata = $this->civi->get_request_args();
     if (!$this->civi->users->check_permission($argdata['args'])) {
-      return $this->civi->users->get_permission_denied();;
+      return $this->civi->users->get_permission_denied();
     }
 
-    // CMW: why do we need this? Nothing that follows uses it.
-    require_once ABSPATH . WPINC . '/pluggable.php';
-
     // Start buffering.
     ob_start();
-    // Now, instead of echoing, shortcode output ends up in buffer.
+    // Now, instead of echoing, Shortcode output ends up in buffer.
     $this->civi->invoke();
     // Save the output and flush the buffer.
     $content = ob_get_clean();
@@ -337,13 +386,13 @@ class CiviCRM_For_WordPress_Shortcodes {
   }
 
   /**
-   * Return a generic display for a shortcode instead of a CiviCRM invocation.
+   * Return a generic display for a Shortcode instead of a CiviCRM invocation.
    *
    * @since 4.6
    *
    * @param int $post_id The containing WordPress post ID.
-   * @param string $shortcode The shortcode being parsed.
-   * @param bool $multiple Boolean flag, TRUE if post has multiple shortcodes, FALSE otherwise.
+   * @param string $shortcode The Shortcode being parsed.
+   * @param bool $multiple Boolean flag, TRUE if post has multiple Shortcodes, FALSE otherwise.
    * @return string $markup Generic markup for multiple instances.
    */
   private function render_multiple($post_id = FALSE, $shortcode = FALSE, $multiple = 0) {
@@ -351,15 +400,17 @@ class CiviCRM_For_WordPress_Shortcodes {
     // Get attributes.
     $atts = $this->get_atts($shortcode);
 
-    // Pre-process shortcode and retrieve args.
+    // Pre-process Shortcode and retrieve args.
     $args = $this->preprocess_atts($atts);
 
-    // Sanity check for improperly constructed shortcode.
-    if ($args === FALSE) {
-      return '<p>' . __('Do not know how to handle this shortcode.', 'civicrm') . '</p>';
+    // Get pathless markup from filter callback.
+    if (empty($args['q'])) {
+      $markup = '';
+      /** This filter is documented in includes/civicrm-shortcodes.php */
+      return apply_filters('civicrm_shortcode_get_markup', $markup, $atts, $args, 'multiple');
     }
 
-    // Get data for this shortcode.
+    // Get data for this Shortcode.
     $data = $this->get_data($atts, $args);
 
     // Sanity check.
@@ -385,7 +436,7 @@ class CiviCRM_For_WordPress_Shortcodes {
     // Access CiviCRM config object.
     $config = CRM_Core_Config::singleton();
 
-    // Do we have multiple shortcodes?
+    // Do we have multiple Shortcodes?
     if ($multiple != 0) {
 
       $links = [];
@@ -474,12 +525,11 @@ class CiviCRM_For_WordPress_Shortcodes {
       $link,
 
       /**
-       * Filter the CiviCRM shortcode more link text.
+       * Filter the CiviCRM Shortcode more link text.
        *
        * @since 4.6
        *
-       * @param str The existing shortcode more link text.
-       * @return str The modified shortcode more link text.
+       * @param str The existing Shortcode more link text.
        */
       apply_filters('civicrm_shortcode_more_link', __('Find out more...', 'civicrm'))
 
@@ -499,12 +549,11 @@ class CiviCRM_For_WordPress_Shortcodes {
       $empowered = sprintf(__('Empowered by %s', 'civicrm'), $civi_link);
 
       /**
-       * Filter the CiviCRM shortcode footer text.
+       * Filter the CiviCRM Shortcode footer text.
        *
        * @since 4.6
        *
-       * @param str $empowered The existing shortcode footer.
-       * @return str $empowered The modified shortcode footer.
+       * @param str $empowered The existing Shortcode footer.
        */
       $footer = apply_filters('civicrm_shortcode_footer', $empowered);
 
@@ -522,14 +571,13 @@ class CiviCRM_For_WordPress_Shortcodes {
     $markup = ob_get_clean();
 
     /**
-     * Filter the computed CiviCRM shortcode markup.
+     * Filter the computed CiviCRM Shortcode markup.
      *
      * @since 4.6
      *
-     * @param str $markup The computed shortcode markup.
+     * @param str $markup The computed Shortcode markup.
      * @param int $post_id The numeric ID of the WordPress post.
-     * @param string $shortcode The shortcode being parsed.
-     * @return str $markup The modified shortcode markup.
+     * @param string $shortcode The Shortcode being parsed.
      */
     return apply_filters('civicrm_shortcode_render_multiple', $markup, $post_id, $shortcode);
 
@@ -562,7 +610,7 @@ class CiviCRM_For_WordPress_Shortcodes {
       return $content;
     }
 
-    // Bail if it has multiple shortcodes.
+    // Bail if it has multiple Shortcodes.
     if (count($this->shortcode_markup[$post->ID]) > 1) {
       return $content;
     }
@@ -580,14 +628,19 @@ class CiviCRM_For_WordPress_Shortcodes {
    * @param int $post_id The numeric ID of the WordPress post.
    * @return string $title The overridden title.
    */
-  public function get_title($title, $post_id) {
+  public function get_title($title, $post_id = 0) {
+
+    // Bail if there is no Post ID.
+    if (empty($post_id)) {
+      return $title;
+    }
 
     // Is this the post?
     if (!array_key_exists($post_id, $this->shortcode_markup)) {
       return $title;
     }
 
-    // Bail if it has multiple shortcodes.
+    // Bail if it has multiple Shortcodes.
     if (count($this->shortcode_markup[$post_id]) > 1) {
       return $title;
     }
@@ -628,32 +681,45 @@ class CiviCRM_For_WordPress_Shortcodes {
   }
 
   /**
-   * Detect and return CiviCRM shortcodes in post content.
+   * Detect and return CiviCRM Shortcodes in post content.
    *
    * @since 4.6
+   * @since 5.44 Made recursive.
    *
    * @param str $content The content to parse.
-   * @return array $shortcodes Array of shortcodes.
+   * @return array $shortcodes Array of Shortcodes.
    */
   private function get_for_post($content) {
 
     // Init return array.
     $shortcodes = [];
 
-    // Attempt to discover all instances of the shortcode.
+    // Attempt to discover all instances of the Shortcode.
     $pattern = get_shortcode_regex();
 
     if (
       preg_match_all('/' . $pattern . '/s', $content, $matches)
       && array_key_exists(2, $matches)
-      && in_array('civicrm', $matches[2])
     ) {
 
-      // Get keys for our shortcode.
+      // Get keys for our Shortcode.
       $keys = array_keys($matches[2], 'civicrm');
 
-      foreach ($keys as $key) {
-        $shortcodes[] = $matches[0][$key];
+      // Add found Shortcodes at this level.
+      if (!empty($keys)) {
+        foreach ($keys as $key) {
+          $shortcodes[] = $matches[0][$key];
+        }
+      }
+
+      // Recurse when nested Shortcodes are found.
+      if (!empty($matches[5])) {
+        foreach ($matches[5] as $match) {
+          if (!empty($match)) {
+            $shortcodes_deeper = $this->get_for_post($match);
+            $shortcodes = array_merge($shortcodes, $shortcodes_deeper);
+          }
+        }
       }
 
     }
@@ -663,12 +729,12 @@ class CiviCRM_For_WordPress_Shortcodes {
   }
 
   /**
-   * Return attributes for a given CiviCRM shortcode.
+   * Return attributes for a given CiviCRM Shortcode.
    *
    * @since 4.6
    *
-   * @param $shortcode The shortcode to parse.
-   * @return array $shortcode_atts Array of shortcode attributes.
+   * @param $shortcode The Shortcode to parse.
+   * @return array $shortcode_atts Array of Shortcode attributes.
    */
   private function get_atts($shortcode) {
 
@@ -684,7 +750,7 @@ class CiviCRM_For_WordPress_Shortcodes {
   }
 
   /**
-   * Preprocess CiviCRM-defined shortcodes.
+   * Preprocess CiviCRM-defined Shortcodes.
    *
    * @since 4.6
    *
@@ -722,7 +788,24 @@ class CiviCRM_For_WordPress_Shortcodes {
         if ($mode == 'preview' || $mode == 'test') {
           $args['action'] = 'preview';
         }
-        $args['q'] = 'civicrm/contribute/transact';
+
+        switch ($action) {
+          case 'transact':
+            $args['q'] = 'civicrm/contribute/transact';
+            break;
+
+          case 'setup':
+            $args['q'] = 'civicrm/contribute/campaign';
+            $args['action'] = 'add';
+            $args['component'] = 'contribute';
+            unset($args['id']);
+            $args['pageId'] = $id;
+            break;
+
+          default:
+            $args['q'] = 'civicrm/contribute/transact';
+            break;
+        }
         break;
 
       case 'pcp':
@@ -784,6 +867,10 @@ class CiviCRM_For_WordPress_Shortcodes {
         elseif ($mode == 'search') {
           $args['q'] = 'civicrm/profile';
         }
+        elseif ($mode == 'map') {
+          $args['q'] = 'civicrm/profile/map';
+          $args['map'] = 1;
+        }
         else {
           $args['q'] = 'civicrm/profile/create';
         }
@@ -800,37 +887,29 @@ class CiviCRM_For_WordPress_Shortcodes {
     }
 
     /**
-     * Filter the CiviCRM shortcode arguments.
+     * Filter the CiviCRM Shortcode arguments.
      *
      * This filter allows plugins or CiviCRM Extensions to modify the attributes
-     * that the 'civicrm' shortcode allows. Injected attributes and their values
+     * that the 'civicrm' Shortcode allows. Injected attributes and their values
      * will also become available in the $_REQUEST and $_GET arrays.
      *
      * @since 4.7.28
      *
-     * @param array $args Existing shortcode arguments.
+     * @param array $args Existing Shortcode arguments.
      * @param array $shortcode_atts Shortcode attributes.
-     * @return array $args Modified shortcode arguments.
      */
-    $args = apply_filters('civicrm_shortcode_preprocess_atts', $args, $shortcode_atts);
-
-    // Sanity check for path
-    if (!isset($args['q'])) {
-      return FALSE;
-    }
-
-    return $args;
+    return apply_filters('civicrm_shortcode_preprocess_atts', $args, $shortcode_atts);
 
   }
 
   /**
-   * Post-process CiviCRM-defined shortcodes.
+   * Post-process CiviCRM-defined Shortcodes.
    *
    * @since 4.6
    *
    * @param array $atts Shortcode attributes array.
    * @param array $args Shortcode arguments array.
-   * @return array|bool $data The array data used to build the shortcode markup, or false on failure.
+   * @return array|bool $data The array data used to build the Shortcode markup, or false on failure.
    */
   public function get_data($atts, $args) {
 
@@ -845,18 +924,16 @@ class CiviCRM_For_WordPress_Shortcodes {
      * Filter the base CiviCRM API parameters.
      *
      * This filter allows plugins or CiviCRM Extensions to modify the API call
-     * when there are multiple shortcodes being rendered.
+     * when there are multiple Shortcodes being rendered.
      *
      * @since 4.7.28
      *
      * @param array $params Existing API params.
      * @param array $atts Shortcode attributes array.
      * @param array $args Shortcode arguments array.
-     * @return array $params Modified API params.
      */
     $params = apply_filters('civicrm_shortcode_api_params', [
-      'version' => 3,
-      'sequential' => '1',
+      'sequential' => 1,
     ], $atts, $args);
 
     // Get the CiviCRM entity via the API.
@@ -868,7 +945,7 @@ class CiviCRM_For_WordPress_Shortcodes {
         $params['id'] = $args['id'];
 
         // Call API.
-        $civi_entity = civicrm_api('contribution_page', 'getsingle', $params);
+        $civi_entity = civicrm_api3('ContributionPage', 'getsingle', $params);
 
         // Set title.
         $data['title'] = $civi_entity['title'];
@@ -887,7 +964,7 @@ class CiviCRM_For_WordPress_Shortcodes {
         $params['id'] = $args['id'];
 
         // Call API.
-        $civi_entity = civicrm_api('event', 'getsingle', $params);
+        $civi_entity = civicrm_api3('Event', 'getsingle', $params);
 
         // Set title.
         switch ($atts['action']) {
@@ -900,23 +977,20 @@ class CiviCRM_For_WordPress_Shortcodes {
 
           case 'info':
           default:
-            $data['title'] = $civi_entity['title'];
+            $data['title'] = '';
+            if (!empty($civi_entity['title'])) {
+              $data['title'] = $civi_entity['title'];
+            }
             break;
         }
 
         // Set text, if present.
         $data['text'] = '';
-        if (isset($civi_entity['summary'])) {
+        if (!empty($civi_entity['summary'])) {
           $data['text'] = $civi_entity['summary'];
         }
-        if (
-          // Summary is not present or is empty.
-          (!isset($civi_entity['summary']) || empty($civi_entity['summary']))
-          &&
-          // We do have a description.
-          isset($civi_entity['description']) && !empty($civi_entity['description'])
-        ) {
-          // Override with description.
+        // Override with "description" if "summary" is empty.
+        if (empty($civi_entity['summary']) && !empty($civi_entity['description'])) {
           $data['text'] = $civi_entity['description'];
         }
 
@@ -934,7 +1008,7 @@ class CiviCRM_For_WordPress_Shortcodes {
         $params['id'] = $args['gid'];
 
         // Call API.
-        $civi_entity = civicrm_api('uf_group', 'getsingle', $params);
+        $civi_entity = civicrm_api3('UFGroup', 'getsingle', $params);
 
         // Set title.
         $data['title'] = $civi_entity['title'];
@@ -949,7 +1023,7 @@ class CiviCRM_For_WordPress_Shortcodes {
         $params['id'] = $atts['id'];
 
         // Call API.
-        $civi_entity = civicrm_api('survey', 'getsingle', $params);
+        $civi_entity = civicrm_api3('Survey', 'getsingle', $params);
 
         // Set title.
         $data['title'] = $civi_entity['title'];
@@ -963,23 +1037,22 @@ class CiviCRM_For_WordPress_Shortcodes {
         break;
 
       default:
-        // Do we need to protect against malformed shortcodes?
+        // Do we need to protect against malformed Shortcodes?
         break;
 
     }
 
     /**
-     * Filter the CiviCRM shortcode data array.
+     * Filter the CiviCRM Shortcode data array.
      *
      * This filter allows plugins or CiviCRM Extensions to modify the data used
-     * to display the shortcode when there are multiple shortcodes being rendered.
+     * to display the Shortcode when there are multiple Shortcodes being rendered.
      *
      * @since 4.7.28
      *
-     * @param array $data The existing shortcode data.
+     * @param array $data The existing Shortcode data.
      * @param array $atts Shortcode attributes array.
      * @param array $args Shortcode arguments array.
-     * @return array $data The modified shortcode data.
      */
     return apply_filters('civicrm_shortcode_get_data', $data, $atts, $args);
 
