@@ -11,7 +11,7 @@ namespace CiviWP {
    */
   class HookTest extends \PHPUnit\Framework\TestCase implements EndToEndInterface {
 
-    public function testFoo() {
+    public function testFoo(): void {
       add_action('civicrm_fakeAlterableHook', 'onFakeAlterableHook', 10, 2);
 
       $arg1 = 'hello';
@@ -19,7 +19,7 @@ namespace CiviWP {
         'foo' => 123,
       );
       $this->assertNotEquals($arg2['foo'], 456);
-      $this->assertNotEquals($arg2['hook_was_called'], 1);
+      $this->assertFalse(isset($arg2['hook_was_called']));
       \CRM_Utils_Hook::singleton()
         ->invoke(
           2,
